@@ -20,18 +20,10 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 
-"""
-sigal - simple static gallery generator
-=======================================
-
-sigal is yet another python script to prepare a static gallery of images:
-
-* resize images, create thumbnails with some options (squared thumbs, ...).
-* generate html pages.
-"""
-
 from __future__ import absolute_import, print_function
 
+import click
+import importlib
 import io
 import locale
 import logging
@@ -39,8 +31,9 @@ import os
 import sys
 import time
 
-from argh import ArghParser, arg
+from click import argument, option
 
+from .compat import server, socketserver, string_types
 from .gallery import Gallery
 from .log import init_logging
 from .pkgmeta import __version__
@@ -50,12 +43,29 @@ from .utils import copy
 _DEFAULT_CONFIG_FILE = 'sigal.conf.py'
 
 
-@arg('path', nargs='?', help='Path of the sample config file')
+@click.group()
+@click.version_option(version=__version__)
+def main():
+    """Sigal - Simple Static Gallery Generator.
+
+    Sigal is yet another python script to prepare a static gallery of images:
+    resize images, create thumbnails with some options, generate html pages.
+
+    """
+    pass
+
+
+@main.command()
+@argument('path', default=_DEFAULT_CONFIG_FILE)
 def init(path):
-    """Copy a sample config file in the current directory."""
+    """Copy a sample config file in the current directory (default to
+    'sigal.conf.py'), or use the provided 'path'."""
+
+    if os.path.isfile(path):
+        print("Found an existing config file, will abort to keep it safe.")
+        sys.exit(1)
 
     from pkg_resources import resource_string
-    path = path or 'sigal.conf.py'
     conf = resource_string(__name__, 'templates/sigal.conf.py')
 
     with io.open(path, 'w', encoding='utf-8') as f:
@@ -63,49 +73,63 @@ def init(path):
     print("Sample config file created: {}".format(path))
 
 
-@arg('source', nargs='?', help='Input directory')
-@arg('destination', nargs='?', help='Output directory (default: _build/)')
-@arg('-f', '--force', help="Force the reprocessing of existing images")
-@arg('-v', '--verbose', help="Show all messages")
-@arg('-d', '--debug', help="Show all message, including debug messages")
-@arg('-c', '--config', help="Configuration file")
-@arg('-t', '--theme', help="Specify a theme directory, or a theme name for "
-     "the themes included with Sigal")
-@arg('-n', '--ncpu', help="Number of cpu to use (default: all)")
-def build(source, destination, debug=False, verbose=False, force=False,
-          config=_DEFAULT_CONFIG_FILE, theme=None, ncpu=None):
-    """Run sigal to process a directory. """
+@main.command()
+@argument('source', required=False)
+@argument('destination', required=False)
+@option('-f', '--force', is_flag=True,
+        help="Force the reprocessing of existing images")
+@option('-v', '--verbose', is_flag=True, help="Show all messages")
+@option('-d', '--debug', is_flag=True,
+        help="Show all message, including debug messages")
+@option('-c', '--config', default=_DEFAULT_CONFIG_FILE, show_default=True,
+        help="Configuration file")
+@option('-t', '--theme', help="Specify a theme directory, or a theme name for "
+        "the themes included with Sigal")
+@option('--title', help="Title of the gallery (overrides the title setting.")
+@option('-n', '--ncpu', help="Number of cpu to use (default: all)")
+def build(source, destination, debug, verbose, force, config, theme, title,
+          ncpu):
+    """Run sigal to process a directory.
 
+    If provided, 'source', 'destination' and 'theme' will override the
+    corresponding values from the settings file.
+
+    """
     level = ((debug and logging.DEBUG) or (verbose and logging.INFO)
              or logging.WARNING)
     init_logging(__name__, level=level)
     logger = logging.getLogger(__name__)
 
-    start_time = time.time()
     if not os.path.isfile(config):
         logger.error("Settings file not found: %s", config)
         sys.exit(1)
+
+    start_time = time.time()
     settings = read_settings(config)
 
-    if source:
-        settings['source'] = os.path.abspath(source)
-    if destination:
-        settings['destination'] = os.path.abspath(destination)
+    for key in ('source', 'destination', 'theme'):
+        arg = locals()[key]
+        if arg is not None:
+            settings[key] = os.path.abspath(arg)
+        logger.info("%12s : %s", key.capitalize(), settings[key])
 
-    logger.info("Input  : %s", settings['source'])
     if not settings['source'] or not os.path.isdir(settings['source']):
         logger.error("Input directory not found: %s", settings['source'])
         sys.exit(1)
 
-    logger.info("Output : %s", settings['destination'])
     if not os.path.relpath(settings['destination'],
                            settings['source']).startswith('..'):
         logger.error("Output directory should be outside of the input "
                      "directory.")
         sys.exit(1)
 
+    if title:
+        settings['title'] = title
+
     locale.setlocale(locale.LC_ALL, settings['locale'])
-    gal = Gallery(settings, theme=theme, ncpu=ncpu)
+    init_plugins(settings)
+
+    gal = Gallery(settings, ncpu=ncpu)
     gal.build(force=force)
 
     # copy extra files
@@ -120,20 +144,41 @@ def build(source, destination, debug=False, verbose=False, force=False,
            'seconds.').format(duration=time.time() - start_time, **gal.stats))
 
 
-@arg('path', nargs='?', default='_build',
-     help='Directory to serve (default: _build/)')
-def serve(path):
+def init_plugins(settings):
+    """Load plugins and call register()."""
+
+    logger = logging.getLogger(__name__)
+    logger.debug('Plugin paths: %s', settings['plugin_paths'])
+
+    for path in settings['plugin_paths']:
+        sys.path.insert(0, path)
+
+    for plugin in settings['plugins']:
+        try:
+            if isinstance(plugin, string_types):
+                mod = importlib.import_module(plugin)
+                mod.register(settings)
+            else:
+                plugin.register(settings)
+            logger.debug('Registered plugin %s', plugin)
+        except Exception as e:
+            logger.error('Failed to load plugin %s: %r', plugin, e)
+
+    for path in settings['plugin_paths']:
+        sys.path.remove(path)
+
+
+@main.command()
+@argument('path', default='_build')
+@option('-p', '--port', help="Port to use", default=8000)
+def serve(path, port):
     """Run a simple web server."""
 
     if os.path.exists(path):
-        from .compat import server, socketserver
-
         os.chdir(path)
-        PORT = 8000
         Handler = server.SimpleHTTPRequestHandler
-        httpd = socketserver.TCPServer(("", PORT), Handler, False)
-
-        print(" * Running on http://127.0.0.1:{}/".format(PORT))
+        httpd = socketserver.TCPServer(("", port), Handler, False)
+        print(" * Running on http://127.0.0.1:{}/".format(port))
 
         try:
             httpd.allow_reuse_address = True
@@ -144,11 +189,4 @@ def serve(path):
             print('\nAll done!')
     else:
         sys.stderr.write("The '%s' directory doesn't exist.\n" % path)
-
-
-def main():
-    parser = ArghParser(description='Simple static gallery generator.')
-    parser.add_commands([init, build, serve])
-    parser.add_argument('--version', action='version',
-                        version='%(prog)s {}'.format(__version__))
-    parser.dispatch()
+        sys.exit(1)
