@@ -23,10 +23,8 @@
 
 from __future__ import absolute_import, print_function
 
-import codecs
 import fnmatch
 import logging
-import markdown
 import multiprocessing
 import os
 import sys
@@ -34,15 +32,16 @@ import zipfile
 
 from collections import defaultdict
 from datetime import datetime
+from functools import partial
 from os.path import isfile, join, splitext
 from PIL import Image as PILImage
 
-from . import image, video
+from . import image, video, signals
 from .compat import UnicodeMixin, strxfrm, url_quote
 from .image import process_image, get_exif_tags
 from .log import colored, BLUE
 from .settings import get_thumb
-from .utils import copy, check_or_create_dir, url_from_path
+from .utils import copy, check_or_create_dir, url_from_path, read_markdown
 from .video import process_video
 from .writer import Writer
 
@@ -80,6 +79,8 @@ class Media(UnicodeMixin):
         self.raw_exif = None
         self.exif = None
         self.date = None
+        self._get_metadata()
+        signals.media_initialized.send(self)
 
     def __repr__(self):
         return "<%s>(%r)" % (self.__class__.__name__, str(self))
@@ -116,14 +117,26 @@ class Media(UnicodeMixin):
             except Exception as e:
                 self.logger.error('Failed to generate thumbnail: %s', e)
                 return
-        return self.thumb_name
+        return url_from_path(self.thumb_name)
+
+    def _get_metadata(self):
+        """ Get image metadata from filename.md: title, description, meta."""
+        self.description = ''
+        self.meta = {}
+        self.title = ''
+
+        descfile = splitext(self.src_path)[0] + '.md'
+        if isfile(descfile):
+            meta = read_markdown(descfile)
+            for key, val in meta.items():
+                setattr(self, key, val)
 
 
 class Image(Media):
     """Gather all informations on an image file."""
 
     type = 'image'
-    extensions = ('.jpg', '.jpeg', '.JPG', '.JPEG', '.png')
+    extensions = ('.jpg', '.jpeg', '.png')
 
     def __init__(self, filename, path, settings):
         super(Image, self).__init__(filename, path, settings)
@@ -136,7 +149,7 @@ class Video(Media):
     """Gather all informations on a video file."""
 
     type = 'video'
-    extensions = ('.MOV', '.mov', '.avi', '.mp4', '.webm', '.ogv')
+    extensions = ('.mov', '.avi', '.mp4', '.webm', '.ogv')
 
     def __init__(self, filename, path, settings):
         super(Video, self).__init__(filename, path, settings)
@@ -201,9 +214,9 @@ class Album(UnicodeMixin):
 
         for f in filenames:
             ext = splitext(f)[1]
-            if ext in Image.extensions:
+            if ext.lower() in Image.extensions:
                 media = Image(f, self.path, settings)
-            elif ext in Video.extensions:
+            elif ext.lower() in Video.extensions:
                 media = Video(f, self.path, settings)
             else:
                 continue
@@ -221,6 +234,8 @@ class Album(UnicodeMixin):
 
             medias.sort(key=key, reverse=settings['medias_sort_reverse'])
 
+        signals.album_initialized.send(self)
+
     def __repr__(self):
         return "<%s>(path=%r, title=%r)" % (self.__class__.__name__, self.path,
                                             self.title)
@@ -243,24 +258,16 @@ class Album(UnicodeMixin):
 
         """
         descfile = join(self.src_path, self.description_file)
+        self.description = ''
+        self.meta = {}
+        # default: get title from directory name
+        self.title = os.path.basename(self.path if self.path != '.'
+                                      else self.src_path)
 
         if isfile(descfile):
-            # Use utf-8-sig codec to remove BOM if it is present
-            with codecs.open(descfile, 'r', 'utf-8-sig') as f:
-                text = f.read()
-
-            md = markdown.Markdown(extensions=['meta'])
-            html = md.convert(text)
-
-            self.title = md.Meta.get('title', [''])[0]
-            self.description = html
-            self.meta = md.Meta.copy()
-        else:
-            self.description = ''
-            self.meta = {}
-            # default: get title from directory name
-            self.title = os.path.basename(self.path if self.path != '.'
-                                          else self.src_path)
+            meta = read_markdown(descfile)
+            for key, val in meta.items():
+                setattr(self, key, val)
 
     def create_output_directories(self):
         """Create output directories for thumbnails and original images."""
@@ -309,7 +316,7 @@ class Album(UnicodeMixin):
 
         if self._thumbnail:
             # stop if it is already set
-            return self._thumbnail
+            return url_from_path(self._thumbnail)
 
         # Test the thumbnail from the Markdown file.
         thumbnail = self.meta.get('thumbnail', [''])[0]
@@ -318,26 +325,30 @@ class Album(UnicodeMixin):
             self._thumbnail = join(self.name, get_thumb(self.settings,
                                                         thumbnail))
             self.logger.debug("Thumbnail for %r : %s", self, self._thumbnail)
-            return self._thumbnail
+            return url_from_path(self._thumbnail)
         else:
             # find and return the first landscape image
             for f in self.medias:
                 ext = splitext(f.filename)[1]
-                if ext in Image.extensions:
-                    im = PILImage.open(f.src_path)
-                    if im.size[0] > im.size[1]:
-                        self._thumbnail = join(self.name, f.thumbnail)
-                        self.logger.debug(
-                            "Use 1st landscape image as thumbnail for %r : %s",
-                            self, self._thumbnail)
-                        return self._thumbnail
+                if ext.lower() in Image.extensions:
+                    try:
+                        im = PILImage.open(f.src_path)
+                    except:
+                        self.logger.error("Failed to open %s", f.src_path)
+                    else:
+                        if im.size[0] > im.size[1]:
+                            self._thumbnail = join(self.name, f.thumbnail)
+                            self.logger.debug(
+                                "Use 1st landscape image as thumbnail for %r :"
+                                " %s", self, self._thumbnail)
+                            return url_from_path(self._thumbnail)
 
             # else simply return the 1st media file
             if not self._thumbnail and self.medias:
                 self._thumbnail = join(self.name, self.medias[0].thumbnail)
                 self.logger.debug("Use the 1st image as thumbnail for %r : %s",
                                   self, self._thumbnail)
-                return self._thumbnail
+                return url_from_path(self._thumbnail)
 
             # use the thumbnail of their sub-directories
             if not self._thumbnail:
@@ -347,7 +358,7 @@ class Album(UnicodeMixin):
                         self.logger.debug(
                             "Using thumbnail from sub-directory for %r : %s",
                             self, self._thumbnail)
-                        return self._thumbnail
+                        return url_from_path(self._thumbnail)
 
         self.logger.error('Thumbnail not found for %r', self)
         return None
@@ -405,9 +416,8 @@ class Album(UnicodeMixin):
 
 class Gallery(object):
 
-    def __init__(self, settings, theme=None, ncpu=None):
+    def __init__(self, settings, ncpu=None):
         self.settings = settings
-        self.theme = theme
         self.logger = logging.getLogger(__name__)
         self.stats = {'image': 0, 'image_skipped': 0,
                       'video': 0, 'video_skipped': 0}
@@ -458,6 +468,12 @@ class Gallery(object):
                 albums[relpath] = album
 
         self.logger.debug('Albums:\n%r', albums.values())
+        signals.gallery_initialized.send(self)
+
+    @property
+    def title(self):
+        """Title of the gallery."""
+        return self.settings['title'] or self.albums['.'].title
 
     def init_pool(self, ncpu):
         try:
@@ -492,19 +508,26 @@ class Gallery(object):
     def build(self, force=False):
         "Create the image gallery"
 
+        if not self.albums:
+            self.logger.warning("No albums found.")
+            return
+
+        log_func = (partial(print, colored('->', BLUE)) if sys.stdout.isatty()
+                    else self.logger.warn)
+
         # loop on directories in reversed order, to process subdirectories
         # before their parent
-        if self.pool:
-            media_list = []
-            processor = media_list.append
-        else:
-            processor = process_file
+        media_list = []
+        processor = media_list.append if self.pool else process_file
 
         try:
             for album in self.albums.values():
                 if len(album) > 0:
+                    log_func(str(album))
                     for files in self.process_dir(album, force=force):
                         processor(files)
+                else:
+                    self.logger.info('Album %r is empty', album)
         except KeyboardInterrupt:
             sys.exit('Interrupted')
 
@@ -521,20 +544,15 @@ class Gallery(object):
             print('')
 
         if self.settings['write_html']:
-            self.writer = Writer(self.settings, theme=self.theme,
-                                 index_title=self.albums['.'].title)
-
+            writer = Writer(self.settings, index_title=self.title)
             for album in self.albums.values():
-                self.writer.write(album)
+                writer.write(album)
+
+        signals.gallery_build.send(self)
 
     def process_dir(self, album, force=False):
         """Process a list of images in a directory."""
 
-        if sys.stdout.isatty():
-            print(colored('->', BLUE), str(album))
-        else:
-            self.logger.warn(album)
-
         for f in album:
             if isfile(f.dst_path) and not force:
                 self.logger.info("%s exists - skipping", f.filename)
