@@ -38,11 +38,11 @@ from copy import deepcopy
 from datetime import datetime
 from PIL.ExifTags import TAGS, GPSTAGS
 from PIL import Image as PILImage
-from PIL import ImageDraw, ImageOps
-from pilkit.processors import Transpose, Adjust
+from PIL import ImageOps
+from pilkit.processors import Transpose
 from pilkit.utils import save_image
 
-from . import compat
+from . import compat, signals
 from .settings import get_thumb
 
 
@@ -97,11 +97,10 @@ def generate_image(source, outname, settings, options=None):
         processor = processor_cls(*settings['img_size'], upscale=False)
         img = processor.process(img)
 
-    # Adjust the image after resizing
-    img = Adjust(**settings['adjust_options']).process(img)
-
-    if settings['copyright']:
-        add_copyright(img, settings['copyright'])
+    # signal.send() does not work here as plugins can modify the image, so we
+    # iterate other the receivers to call them with the image.
+    for receiver in signals.img_resized.receivers_for(img):
+        img = receiver(img, settings=settings)
 
     outformat = img.format or original_format or 'JPEG'
     logger.debug(u'Save resized image to {0} ({1})'.format(outname, outformat))
@@ -152,28 +151,16 @@ def process_image(filepath, outpath, settings):
                            fit=settings['thumb_fit'], options=options)
 
 
-def add_copyright(img, text):
-    """Add a copyright to the image."""
-
-    draw = ImageDraw.Draw(img)
-    draw.text((5, img.size[1] - 15), '\xa9 ' + text)
-
-
 def _get_exif_data(filename):
     """Return a dict with EXIF data."""
 
     img = PILImage.open(filename)
     exif = img._getexif() or {}
-    data = dict((TAGS.get(t, t), v) for (t, v) in exif.items())
+    data = {TAGS.get(tag, tag): value for tag, value in exif.items()}
 
     if 'GPSInfo' in data:
-        gps_data = {}
-
-        for tag in data['GPSInfo']:
-            gps_data[GPSTAGS.get(tag, tag)] = data['GPSInfo'][tag]
-
-        data['GPSInfo'] = gps_data
-
+        data['GPSInfo'] = {GPSTAGS.get(tag, tag): value
+                           for tag, value in data['GPSInfo'].items()}
     return data
 
 
@@ -189,7 +176,8 @@ def dms_to_degrees(v):
 def get_exif_tags(source):
     """Read EXIF tags from file @source and return a tuple of two dictionaries,
     the first one containing the raw EXIF data, the second one a simplified
-    version with common tags."""
+    version with common tags.
+    """
 
     logger = logging.getLogger(__name__)
 
@@ -199,7 +187,7 @@ def get_exif_tags(source):
     try:
         data = _get_exif_data(source)
     except (IOError, IndexError, TypeError, AttributeError):
-        logger.warning(u'Could not read EXIF data from {0}'.format(source))
+        logger.warning(u'Could not read EXIF data from %s', source)
         return (None, None)
 
     simple = {}
@@ -237,8 +225,8 @@ def get_exif_tags(source):
             else:
                 simple['datetime'] = dt
         except (ValueError, TypeError) as e:
-            msg = u'Could not parse DateTimeOriginal of %s: %s' % (source, e)
-            logger.warning(msg)
+            logger.warning(u'Could not parse DateTimeOriginal of %s: %s',
+                           source, e)
 
     if 'GPSInfo' in data:
         info = data['GPSInfo']
@@ -248,15 +236,17 @@ def get_exif_tags(source):
         lon_ref_info = info.get('GPSLongitudeRef')
 
         if lat_info and lon_info and lat_ref_info and lon_ref_info:
-            lat = dms_to_degrees(lat_info)
-            lon = dms_to_degrees(lon_info)
+            try:
+                lat = dms_to_degrees(lat_info)
+                lon = dms_to_degrees(lon_info)
+            except (ZeroDivisionError, ValueError):
+                logger.warning('Failed to read GPS info for %s', source)
+                lat = lon = None
 
-            if lat_ref_info != 'N':
-                lat = 0 - lat
-
-            if lon_ref_info != 'E':
-                lon = 0 - lon
-
-            simple['gps'] = {'lat': lat, 'lon': lon}
+            if lat and lon:
+                simple['gps'] = {
+                    'lat': - lat if lat_ref_info != 'N' else lat,
+                    'lon': - lon if lon_ref_info != 'E' else lon,
+                }
 
     return (data, simple)
